'''
	CD_Data.py
		A simple class for hold a CD's information.

	Copyright 2004 Ron Kuslak, <rds@rdsarts.com>
		all rights reserved
	
	This is Free-as-in-love software licensed under the 'Screw General Public License.'
	Use it how you want. make a note in the docs/source of where you got it, you have no
	warrenty, and If you feel indebted, when you see me have some sex.
'''

import rox
from debug_utils import *
from rox import choices
import cd_fileio, os

_ = rox.i18n.translation(os.path.join(rox.app_dir, 'Messages'))

class CdData(object):
	def __init__(self, app_name = 'RoxCd'):
		self.app_name = app_name
		self.clear()

	def clear(self):
		# Clears out all values in the class.
		self.artist = _('<unknown>')
		self.album = _('<unknown>')
		self.genre = ''
		# This should really get set to the local time's year.
		self.year = 1700
		self.tracklist = []
		self.modified = False
		self.save_on_change = False

		# CD number is the number we use to determine a filename.
		self.cd_number = None

	def set_artist(self, artist):
		self.artist = artist

	def set_album(self, album):
		self.album = album

	def set_genre(self, genre):
		self.genre = genre

	def set_year(self, year):
		if type(year) == str:
			try:
				year = int(year)
			except ValueError:
				year = 0
		self.year = year

	def set_tracklist(self, tracklist):
		# Sets 'self.tracklist' to 'tracklist.' Forces it to be 'tracks' number of tracks.
		self.tracklist = []
		for track in tracklist:
			# XXXXXXX: this is not right. Fix. Raise bells und whistles.
			try:
				track = track.encode('utf-8')
			except:
				track = _('<Unknown>')
			self.tracklist.append(track)

	def set_track_name(self, number, name):
		if number <= len(self.tracklist):
			self.tracklist[number - 1] = name

	def set_cd_number(self, cd_number):
		# Sets the CD Number and filename. Passing 'None' clears both.
		try:
			self.cd_number = int(cd_number)
		except:
			self.cd_number = None
	
	def set_number_of_tracks(self, tracks):
		# Sets tracklist to contain 'tracks' number of tracks. 1 as first track.

		num_tracks = len( self.get_tracklist() )
		# debugger('Entered with ', self.get_tracklist())
		if num_tracks > tracks:
			# debugger('Removing tracks...')
			self.tracklist = self.tracklist[:tracks - 1]
		elif num_tracks < tracks:
			# debugger('Adding tracks...')
			for track_num in range( len(self.tracklist), tracks):
				self.tracklist.append( 'Track ' + str(track_num + 1) )
		# debugger('Left with ', self.get_tracklist())

	def set_modified(self, modified):
		# Called to change the modified state.
		self.modified = modified

	# get_* ######################################################################

	def get_artist(self):
		return self.artist

	def get_album(self):
		return self.album

	def get_artist(self):
		return self.artist

	def get_genre(self):
		return self.genre

	def get_year(self):
		return str(self.year)

	def get_year_as_int(self):
		return self.year

	def get_tracklist(self):
		return self.tracklist

	def get_cd_number(self):
		return self.cd_number

	# save/load ###################################################################

	def load(self):
		'''
			Attempts to load CD info from local filesystem.
			WILL SET ONLY THE SAME TRACKLIST SIZE AS IT HAS.
			CALL .set_number_of_tracks() FIRST!!
		'''
		self.loaded = False
		if self.cd_number:
			filename = choices.load(self.app_name, str(self.cd_number) + '.xml')
			if filename:
				try:
					# Returns (title, artist, genre, year, tracklist) or raises a exception
					cd_data = cd_fileio.load_cd(filename)
				except:
					return False
				self.set_album(cd_data[0])
				self.set_artist(cd_data[1])
				self.set_genre(cd_data[2])
				self.set_year(cd_data[3])
				temp_tracks = len( self.get_tracklist() )
				self.set_tracklist(cd_data[4])
				self.set_number_of_tracks(temp_tracks)
				self.loaded = True
		return self.loaded

	def save(self):
		# Saves current tracklist to file if CHOICESDIR is set and CD is inserted.
		temp = False
		if self.cd_number:
			try:
				filename = str(self.cd_number) + '.xml'
			except IOError:
				# No CD. :P
				return temp
			filename = choices.save(self.app_name, filename)

			if filename:
				album =		self.get_album()
				artist =	self.get_artist()
				genre = 	self.get_genre()
				year =		self.get_year()
				tracklist = 	self.get_tracklist()

				temp = cd_fileio.save_cd(filename, album, artist, genre, year, tracklist)
		return temp
