"""
	cd_saveload.py
		XML database backend for RoxCD

	Copyright 2003 Ron Kuslak. <rds@rdsarts.com>
	All rights reserved.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from xml.dom import minidom
from debug_utils import *
import os

# True globals
APP_NAME= 'cd_loadsave'
APP_VER = '1.0'

def save_cd(filename, album, artist, genre, year, tracklist):
	''' Saves current tracklist to file if CHOICESDIR is set and CD is inserted. '''
	if filename == None:
		return False

	cd_tree = minidom.Document()
	cd_tree.appendChild(cd_tree.createComment('Creator: ' + APP_NAME + ' ' + APP_VER + '.'))
	root_node = cd_tree.createElement('album')
	root_node.setAttribute('title', album.encode('utf-8'))
	root_node.setAttribute('artist',artist.encode('utf-8'))
	root_node.setAttribute('genre', genre.encode('utf-8') )
	root_node.setAttribute('year',  year.encode('utf-8') )

	track_number = 1
	for iter in tracklist:
		track_node = cd_tree.createElement('track')
		track_node.setAttribute('number', str(track_number))
		track_node.appendChild(cd_tree.createTextNode(iter.encode('utf-8')))
		root_node.appendChild(track_node)
		track_number += 1

	cd_tree.appendChild(root_node)
	FD = open(filename, 'w')
	FD.write(cd_tree.toprettyxml())
	FD.close()
	return True
	
def load_cd(file):
	''' Loads cd track list from xml file in Choices dir. '''
	title, artist, genre, year, tracklist = '', '', '', 0, []

	if file != None and os.path.exists(file):
		xmldoc=minidom.parse(file)
		albums = xmldoc.getElementsByTagName('album')
		for album in albums:
			tracks = album.getElementsByTagName('track')
			title = album.getAttribute('title')
			artist = album.getAttribute('artist')
			genre = album.getAttribute('genre')
			year = '0%s'%album.getAttribute('year')
			for iter in range(len(tracks)):
				tracklist.append('<Unknown>')
			for iteration in tracks:
				track_number = '0' + iteration.getAttribute('number')
				try:
					track_number = int(track_number)
					temp = iteration.childNodes
					for iter in temp:
						# tracklist.append( iter.data.strip() )
						tracklist[track_number - 1] = iter.data.strip()
				except:
					pass
	else:
		# We don't have a file. Pass a empty tracklist as a indication. (No CD will have 0 tracks. Not even data)
		raise IOError, 'Could not read CD data from %s.'%file
	return title, artist, genre, year, tracklist

def init_cd_values(tracks):
	title = 'Unknown'
	artist = 'Unknown'
	genre = 'Unknown'
	year = '1700'
	tracklist = []
	for iter in range(1,tracks):
		temp = 'Track ' + str(iter)
		tracklist.append(temp)
	return title, artist, genre, year, tracklist

