#!/usr/bin/env python

'''
	rename_dialog.py
		A dialog for renaming tracks for RoxCD
	Copyright 2004 Ron Kuslak, <rds@rdsarts.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

'''
import rox
from rox import g, saving

class RenameDialog(rox.Dialog):
	def __init__(self, parent, cd_data, playlist):
		track_number, track_name = playlist.get_selected_track_number()
		if not track_number:
			# No track selected.
			raise OSError

		rox.Dialog.__init__(self, _('Rename track %d')%track_number, parent, g.DIALOG_DESTROY_WITH_PARENT)
		name = g.Entry()
		image = g.Image()
		OK_CLICKED = 0
		RES_CLICKED = 1
		QUIT_CLICKED = 2

		image.set_from_pixbuf(saving.image_for_type('audio/x-wav'))
		ok = rox.ButtonMixed(g.STOCK_OK,		_('OK'))
		res = rox.ButtonMixed(g.STOCK_UNDO,	_('Revert'))

		def ok_close():
			cd_data.set_track_name(track_number, name.get_text())
			playlist.set_track_name(track_number, name.get_text())
			self.destroy()
		
		def key_press_handler(widget, event):
			if event.keyval == g.keysyms.Escape:
				self.response(QUIT_CLICKED)
				return g.TRUE

		def changed(button):
			res.set_sensitive(g.TRUE)

		def restore():
			name.set_text(track_name)
			res.set_sensitive(g.FALSE)

		def response_handler(widget, response):
			print "Got response %d"%response
			if response == OK_CLICKED:
				ok_close()
			elif response == RES_CLICKED:
				restore()
			else:
				# Recieved "unhandled" signal, assuming is close
				self.destroy()

		name.connect_after('changed', changed)
		name.connect('activate', lambda *args: self.response(OK_CLICKED))
		self.connect('response', response_handler)
		self.connect('key-press-event', key_press_handler)
		# dialog.connect('destroy', ok_close, self)
		restore()

		self.add_action_widget(res, RES_CLICKED)
		self.add_action_widget(ok, OK_CLICKED)

		self.vbox.pack_start(image, padding = 4)
		self.vbox.pack_start(name, expand = g.FALSE)

		res.set_sensitive(g.FALSE)