'''
	Ripper_lib
		backend code for the Ripper_dialog

	Copyright 2004 Ron Kuslak <rds@rdsarts.com>
	Copyright 2004 Ken Hayber <khayber@socal.rr.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import string, os, sys, popen2, gtk, signal
from threading import Thread

APP_NAME = "ripper_lib_NG.py"
APP_VER = "0.1"

CODEPATH_UNKNOWN	= 0
CODEPATH_ENCODER	= 1
CODEPATH_CDPARANOIA	= 2
CODEPATH_CDDA2WAV	= 3
USE_GTK = True

# Translation for removing all the needless cruft from the encoder lines.
replace = string.punctuation
for char in ('.', '%'):
	temp = replace.rfind(char)
	replace = replace[0:temp] + replace[temp + 1:]
replace += string.ascii_letters
temp = ''
	
# Blank space for translation to place with.
for dummy in range(0, len(replace)):
		temp += ' '
replace = string.maketrans(replace, temp)

def debugger(*text):
	# Prints a message to STDERR. If your running ROX-Session, it'll show up to the user. ;)
	print >>sys.stderr,"%s %s - %s"%(APP_NAME, APP_VER, text)

def readline(f_descriptor):
	ret_line = ''

	# Prime the line, and start it up.
	char_read = f_descriptor.read(1)
	ret_line += char_read

	while char_read != '' and char_read != '\n' and char_read != '\r':
		# Loop until newline, part of a newline, or no string == EOF
		char_read = f_descriptor.read(1)
		ret_line += char_read

	return ret_line

def update_percent(widget, text = None, percent = None):
	# A convinience wrapper of the GTK update stuffs. So it's all in one place. :P

	if not USE_GTK: # couldn't init threads
		return

	if percent > 100: percent = 100
	if percent < 0: percent = 0
	gtk.threads_enter()
	try:
		if percent:
			percent = float( str( percent ) ) / 100
			widget.set_progress(text, percent)
		else:
			# Pulsing removes text, so that's one less command to bother with...
			widget.pulse()
	except:
		debugger("failure setting widget with percent. Passing...")
	gtk.threads_leave()

def pop_cmds(widget):
	gtk.threads_enter()
	temp = widget.pop_cmds()
	gtk.threads_leave()
	return temp

class main_engine(Thread):
	def __init__(self, queue, finished, rip_widget, enc_widget):
		Thread.__init__(self)
		self.done = False
		self.percent = None
		self.ripper = None
		self.queue_widget = queue
		self.finished_widget = finished
		self.rip_widget = rip_widget
		self.enc_widget = enc_widget

	def run(self):
		while self.queue_widget.number_of_tracks():
			# If calling process wants us gone, just quit...
			if self.done:
				# XXX: Should pass something or destroy savebox to stop thread-holding bug in savebox
				# XXX: Should move ^^^ outside this loop, and check for self.done and do our work there.
				return

			# debugger(self.queue_widget.number_of_tracks())
		 	track_name, enc_cmd, rip_cmd, cd_filename = pop_cmds(self.queue_widget)


			# Set codepath for ripper
			rip_codepath = CODEPATH_UNKNOWN
			if rip_cmd.rfind('cdparanoia') != -1:
				rip_codepath = CODEPATH_CDPARANOIA
			if rip_cmd.rfind('cdda2wav') != -1:
				rip_codepath = CODEPATH_CDDA2WAV

			for cmd, widget_text, codepath, widget in [[rip_cmd, track_name, rip_codepath, self.rip_widget], 
									[enc_cmd, track_name, CODEPATH_ENCODER, self.enc_widget]]:
				if not self.done:
					gtk.threads_enter()
					widget.set_text(widget_text)
					gtk.threads_leave()
					self.ripper = self.pipe_cmd(cmd, codepath, widget, '%s%%')
					if not self.done: update_percent(widget, 'Completed', 100)

			if os.path.exists(cd_filename):
				os.remove(cd_filename)

			gtk.threads_enter()
			self.finished_widget.push_cmds(track_name, '', '', '')
			gtk.threads_leave()

		# Turn out the lights when we leave...
		gtk.threads_enter()
		self.rip_widget.set_sensitive(gtk.FALSE)
		self.enc_widget.set_sensitive(gtk.FALSE)
		gtk.threads_leave()

	def pipe_cmd(self, cmd, codepath, widget = None, widget_text = ''):
		total, start_sector, end_sector = None, None, None
		percent = 0
		cmd_thread = popen2.Popen4(cmd)
		from_cmd = cmd_thread.fromchild
		writes = 0
		line = readline(from_cmd)
		while line != '' and not self.done:

			if codepath == CODEPATH_ENCODER or codepath == CODEPATH_CDDA2WAV:
				line = line.translate(replace).strip()
				percent = line.split('%')[0]				# Find % sign
				percent = percent.strip().split(' ')[-1].strip()	# Read only what's directly before it
				try:
					percent = float (percent)
					percent = int(percent)
				except:
					try:
						percent = int(percent)
					except:
						percent = None

			if codepath == CODEPATH_CDPARANOIA:
				# engine for the CDparanoia
				spliter = line.split('sector')
				if total:
					try:
						state = line.split('@')
						state = state[0].split('[')[1]
						if state[:5] == 'wrote':
							writes += 1
							percent = int( (float(writes) / float(total)) * 100)
					except:
						pass
				elif end_sector:
					total = long(end_sector) - long(start_sector)
				elif spliter[0][:7] == 'Ripping':
					temp = spliter[1].split('(')[0].strip()
					start_sector = int(temp)
				elif spliter[0].strip()[:2] == 'to':
					temp = spliter[1].split('(')[0].strip()
					end_sector = int(temp)
			else:
				# CODEPATH_UNKNOWN - This could be made more exciting, if someone were interested...
				writes += 1
				if writes > 250:
					writes = 0
					update_percent(widget)

			if widget and percent != self.percent:
				try:
					update_percent(widget, widget_text%percent, percent)
				except:
					update_percent(widget, 'Working...', percent)
				self.percent = percent
			line = readline(from_cmd)
		# Send SIGKILL to process group of ripper/encoder. Should get it to stop.. Should.
		os.killpg(cmd_thread.pid, signal.SIGKILL)
		from_cmd.close()
		cmd_thread.wait()

	def stop_end(self):
		self.done = True
