from cd_logic import cd_logic
from rox import g

def stock_image(stock_id, size = g.ICON_SIZE_LARGE_TOOLBAR):
	temp = g.Image()
	temp.set_from_stock(stock_id, size)
	return temp

class Genre(g.Combo):
	# Sane-size combo-box for 
	def __init__(self):
		g.Combo.__init__(self)
		self.set_size_request(1, -1)

class Year(g.SpinButton):
	# A SpinButton custom-tooled for years.
	def __init__(self, adjustment = None, climb_rate = 0.0, digits = 0):
		g.SpinButton.__init__(self, adjustment, climb_rate, digits)

		# We'll all be long gone before this needs to be updated. Let the people in 9000 fix it. /Like suckers/.
		self.set_range(1700, 9000)	
		self.set_increments(1.0, 0.0)		

class TimeScale(g.HScale):
	def __init__(self):
		g.HScale.__init__(self)
		self.set_draw_value(g.FALSE)
		self.connect('adjust-bounds', self.time_scale_moved)
		self.connect('move-slider', self.time_scale_key)

	def time_scale_moved(self, range, value):
		"""
			Called when the scroll-bar is moved by the user.
			Advances the CD in time to where the user has moved
			the scroll bar.
		"""
		value = int(range.get_value())
		track_time = cd_logic.get_track_time()

		if value < 0:
			value = 0
		cd_logic.jump(value - track_time)

	def time_scale_key(self, range, key):
		value = range.get_value()

		if key == g.SCROLL_STEP_UP or key == g.SCROLL_STEP_LEFT:
			# - 5
			range.set_value(value - 5)
		if key == g.SCROLL_STEP_DOWN or key == g.SCROLL_STEP_RIGHT:
			# + 5
			range.set_value(value + 5)
		if key == g.SCROLL_PAGE_UP:
			# - 30
			range.set_value(value - 30)
		if key == g.SCROLL_PAGE_DOWN:
			# + 30
			range.set_value(value + 30)
		if key == g.SCROLL_START:
			# Go to start of track
			range.set_value(0)
		if key == g.SCROLL_END:
			# Play next track
			range.set_value(0)
			self.next()

	def set_sensitive(self, value):
		# Makes everything look nice when insensitive.
		if value == False:
			# I'm insensitive YOU INSENSITIVE CLOD!
			self.set_range(0, 1)
			self.set_value(0.0)
		g.HScale.set_sensitive(self, value)

class Entry(g.Entry):
	def __init__(self, max = 255):
		# Like the gtk 'Entry' widget, but will save CD data on every 
		# change... and has a humane limit on size. ;)
		g.Entry.__init__(self, max)

class Toolbar(g.Toolbar):
	def __init__(self, parent):
		'''
			A toolbar set up for the CD player. Should set a sane
			size-request, so it does not needlessly enlarge the window.
			Call with parent class' instance.
		'''
		g.Toolbar.__init__(self)
		self.play_icon = stock_image(g.STOCK_GO_FORWARD)
		self.stop_icon = stock_image(g.STOCK_STOP)
		self.media_buttons = []

		self.set_size_request(200, -1)

		self.append_widgets(parent)

	def append_widgets(self, parent):
		# Appends all the buttons to the toolbar
		ADD_SPACE = None

		for toolbar_item in [ [ _('Close'), None, g.STOCK_CLOSE, parent.close_evt, False],
			[ _('Options'), None, g.STOCK_PREFERENCES, parent.show_options, False],
			[ ADD_SPACE ],
			[ _('Previous'), None, g.STOCK_GOTO_FIRST, parent.prev, True],
			[ _('Play/Pause'), None, self.play_icon, parent.toolbar_play, True],
			[ _('Next'), None, g.STOCK_GOTO_LAST, parent.next, True],
			[ _('Stop'), None, self.stop_icon, parent.stop, True],
		]:

			if len(toolbar_item) > 1:
				label, tooltip, image, callback, append_button = toolbar_item
				if type(image) != g.Image:
					image = stock_image(image)
				temp = self.append_item(label, tooltip, 'Private', image, callback)
				if append_button:
					self.media_buttons.append(temp)
			else:
				self.append_space()

		del ADD_SPACE

	def set_stop_icon(self, show_stop):
		# Sets the icon for the stop button. If true, it appears as a stop. Otherwise, it shows GO_UP
		# XXX: Change to media buttons.
		icon_size = g.ICON_SIZE_LARGE_TOOLBAR
		if show_stop:
			self.stop_icon.set_from_stock(g.STOCK_STOP, icon_size)
		else:
			self.stop_icon.set_from_stock(g.STOCK_GO_UP, icon_size)

	def set_play_icon(self, show_stop):
		# Sets the icon for the play button. If true, it appears as a play. Otherwise, it shows pause.
		# XXX: Change to media buttons.
		icon_size = g.ICON_SIZE_LARGE_TOOLBAR
		if show_stop:
			self.play_icon.set_from_stock(g.STOCK_GO_FORWARD, icon_size)
		else:
			self.play_icon.set_from_stock(g.STOCK_GO_FORWARD, icon_size)

	def set_sensitive(self, value):
		# Sets all buttons except close and options sensitivity to 'value'
		for widget in self.media_buttons:
			widget.set_sensitive(value)

class PlayList(g.Frame):
	def __init__(self):
		g.Frame.__init__(self)

		self.renderer  = g.CellRendererText()
		self.liststore = g.ListStore(int, str, str, int)
		self.data_init()
		self.ui_init()

	def data_init(self):
		# Zero out all data for the tracklist. SHOULD BE CALLED BY #clear()!!!
		self.tracks = 0
		self.liststore.clear()

	def ui_init(self):
		# Track number(int), track name(string), font(string), 
		scrolled_win = g.ScrolledWindow()
		self.treeview = g.TreeView(self.liststore)
		self.col = g.TreeViewColumn(None, self.renderer, text=1)
		self.col.add_attribute(self.renderer, 'background-set', 3)
		self.col.add_attribute(self.renderer, 'font', 2)

		self.treeview.insert_column(self.col, 0)
		scrolled_win.set_policy(g.POLICY_AUTOMATIC, g.POLICY_ALWAYS)
		scrolled_win.add(self.treeview)
		self.add(scrolled_win)

		self.treeview.set_rules_hint(True)
		self.treeview.set_headers_visible(g.FALSE)

	def set_color(self, color):
		self.renderer.set_property('background', color)

	def set_highlighted(self, track):
		# Sets the track 'highlighted' as playing in widget.
		# print "Setting highlighted track to %d"%track
		for iter in range(self.tracks):
			self.liststore[iter][2] = ''
			self.liststore[iter][3] = g.FALSE
		if type(track) == int and track > 0 and track <= self.tracks:
			self.liststore[track - 1][2] = 'bold'
			self.liststore[track - 1][3] = g.TRUE
			self.treeview.scroll_to_cell((track - 1))

	def set_tracks(self, tracks):
		# sets track list to matrix 'tracks'
		self.clear()
		for track_name in tracks:
			self.tracks += 1
			self.liststore.append([int(self.tracks), str(track_name), '', 0])
		return

	def clear(self):
		self.data_init()

	def update_names(self):
		return

	def get_selected(self):
		# Returns the currently selected entry.
		try:
			temp, iter = self.treeview.get_selection().get_selected()
			temp = self.liststore.get_path(iter)[0]
			return temp
		except:
			return -1

	def get_selected_track_number(self):
		selected = self.get_selected()
		if selected > -1:
			list_item = self.liststore[selected]
			return((list_item[0], list_item[1]))
		return ((None, None))

	def set_selected(self, where):
		# Places cursor on row 'where' of PlayList
		if where > self.tracks - 1:
			where = self.tracks - 1
		self.treeview.set_cursor((where))
		self.treeview.scroll_to_cell((where))	
	
	def set_track_name(self, track_number, track_name):
		for track in range(self.tracks):
			if self.liststore[track][0] == track_number:
				self.liststore[track][1] = track_name
		self.update_names()

	def get_track_name(self, track_number):
		# Returns the name of the track in 
		return self.liststore[track_number - 1][1]
